/* Copyright 2008 Daniel Vokó <vokoda@gmail.com>
 * 
 * This file is part of Sagittarius.
 *
 * Sagittarius is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * Sagittarius is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Sagittarius.  If not, see <http://www.gnu.org/licenses/>.
 */

using System;
using System.Collections.Specialized;
using System.IO;
using Sagittarius.Exiftool;

namespace Sagittarius.Metadata
{
	/// <summary>
	/// Represents the metadata information of an image file
	/// Contains MetadataSection-s
	/// </summary>
	public class XmpMetadata
	{
		//private MetadataSection xmpIptcSection, xmpExifSection;
		private string filename;
		private IptcSection iptcSection;
		private ExifSection exifSection;
		private ExiftoolWrapper exiftoolWrapper;

#region konstruktorok

		public XmpMetadata(string filename,ExiftoolWrapper exiftoolWrapper)
		{
			//Console.WriteLine("Entering XmpMetadata constructor");
			this.filename=filename;
			this.exiftoolWrapper=exiftoolWrapper;
			CreateSections();
			string metadataString = exiftoolWrapper.GetXmpTags(filename);
			string[] metadataLines = metadataString.Split(new char[] {'\n'});// split it to lines
			if(metadataLines.Length<=1)
			{
				exiftoolWrapper.CreateXmp(filename);
				metadataString = exiftoolWrapper.GetXmpTags(filename);
				metadataLines = metadataString.Split(new char[] {'\n'});// split it to lines
			}
			ParseXmp(metadataLines);
			
			//Console.WriteLine("Leaving XmpMetadata constructor");
		}
		
		public XmpMetadata()
		{
			CreateSections();
		}

#endregion

		private void CreateSections()
		{
			// Creating of the XMP-IPTC section
			iptcSection = new IptcSection();
			// Creating of the XMP-EXIF section
			exifSection = new ExifSection();
		}
		/// <summary>
		/// Parses the section-header line of Exiftool output
		/// </summary>
		/// <param name="sectionHeadString">
		/// A <see cref="System.String"/>
		/// </param>
		/// <returns>
		/// A <see cref="System.String"/>
		/// </returns>
		public static string ParseSectionHead (string sectionHeadString)
		{
			return sectionHeadString.Substring(Strs.SECT_HEAD_START_PATH.Length,sectionHeadString.Length-Strs.SECT_HEAD_START_PATH.Length-Strs.SECT_HEAD_END_PATH.Length);
		}

		/// <summary>
		/// Creates a MetadataElement out of a line of the output of Exiftool
		/// </summary>
		/// <param name="metadataLine">
		/// A line of the exitool-output <see cref="System.String"/>
		/// </param>
		/// <returns>
		/// A <see cref="MetadataElement"/>
		/// </returns>
		private static MetadataElement ParseElement(string metadataLine )
		{
			string[] parts = metadataLine.Split(new string[] {": "},StringSplitOptions.None);
			return new MetadataElement(parts[0].Trim(),parts[1].Trim());
		}

		/// <summary>
		/// This method writes the XMP-IPTC information back to the image-file
		/// </summary>
		/// <param name="filename">
		/// Filename string <see cref="System.String"/>
		/// </param>
		/// <returns>
		/// returns the output of exiftool write commmand <see cref="System.String"/>
		/// </returns>
		public string WriteMetadata(bool createBackup)
		{
			//Console.WriteLine("Entering XmpMetadata.WriteIptc");
			string args = "";
			//MetadataSection xmpIptcSect = GetSection(Strs.XMP_IPTC_SECT_NAME); //gets the XMP-IPTC section
			foreach(MetadataElement element in iptcSection.Elements)
			{
				string shortKey = element.Key.Replace(" ", "");// remove the spaces of the key names (exiftool doesn't like them)
				if (element.Key.Contains("Creator ")) // in the case of the CreatorContactInfo XMP-section, things are a bit tricky, cause Exiftool expects the standard names, although if reading the exiftool output uses other namings
				{
					switch(shortKey)
					{
						case "CreatorCity":
						{
							shortKey = "CreatorContactInfoCiAdrCity";
							break;
						}
						case "CreatorCountry":
						{
							shortKey = "CreatorContactInfoCiAdrCtry";
							break;
						}
						case "CreatorAddress":
						{
							shortKey = "CreatorContactInfoCiAdrExtadr";
							break;
						}
						case "CreatorPostalCode":
						{
							shortKey = "CreatorContactInfoCiAdrPcode";
							break;
						}
						case "CreatorRegion":
						{
							shortKey = "CreatorContactInfoCiAdrRegion";
							break;
						}
						case "CreatorWorkEmail":
						{
							shortKey = "CreatorContactInfoCiEmailWork";
							break;
						}
						case "CreatorWorkTelephone":
						{
							shortKey = "CreatorContactInfoCiTelWork";
							break;
						}
						case "CreatorWorkURL":
						{
							shortKey = "CreatorContactInfoCiUrlWork";
							break;
						}
						default:
						{
							break;
						}
					}
				}
						args += " -XMP:" + shortKey + "=" + "'" + element.Value + "'";
			}
			//Console.WriteLine(args);
			//Console.WriteLine("Exiting XmpMetadata.WriteIptc");
			//return Exiftool.WriteXmp(filename,args);
			string output;
			if(createBackup)
			{
			output = exiftoolWrapper.WriteXmpTags('"'+filename+'"',args);
			output+="\n"+ exiftoolWrapper.SynchronizeToXmp(filename,"");
			}
			else
			{
			output = exiftoolWrapper.WriteXmpTags('"'+filename+'"'+" -overwrite_original ",args);
			output+="\n"+ exiftoolWrapper.SynchronizeToXmp(filename," -overwrite_original ");
			}

			return output;
		}
		
		/// <summary>
		/// Parses the exiftool xmp-reading command output
		/// </summary>
		/// <param name="metadataString">
		/// the exiftoolxmp-reading command output <see cref="System.String"/>
		/// </param>
		private void ParseXmp (string[] metadataLines)
		{
			//Console.WriteLine("Enterint XmpMetadata.ParseXmp");
			MetadataSection input = new MetadataSection(); 
			for (int i = 0; i<metadataLines.Length-1;i++) // put each line to a MetadataSection as a MetadataElement
			{
				try//ezmiértvanittnemtudom
				{
					input.AddElement(ParseElement(metadataLines[i]));
				}
				catch
				{}
			}
			foreach(MetadataElement iptcElement in iptcSection.Elements)
			{
				iptcElement.Value=input.GetValue(iptcElement.Key);
			}
			foreach(MetadataElement exifElement in exifSection.Elements)
			{
				exifElement.Value=input.GetValue(exifElement.Key);
			}
			//Console.WriteLine("Leaving XmpMetadata.ParseXmp");
		}
		
		public XmpMetadata Copy()
		{
			//Console.WriteLine("Entering XmpMetadata.Copy");
			XmpMetadata result = new XmpMetadata();
			result.XmpExifSection=this.XmpExifSection.Copy();
			result.XmpIptcSection=this.XmpIptcSection.Copy();
			result.Filename=filename;
			result.exiftoolWrapper=exiftoolWrapper;
			//Console.WriteLine("Leaving XmpMetadata.Copy");
			return result;
		}

		public IptcSection XmpIptcSection
		{
			get { return iptcSection; }
			set { iptcSection=value; }
		}
		
		public string Filename
		{
			get { return filename; }
			set { filename = value; }
		}

		public ExifSection XmpExifSection
		{
			get { return exifSection; }
			set { exifSection = value; }
		}

		public override string ToString()
		{
			return exifSection.ToString()+ "\r\n"+ iptcSection.ToString();
		}
	}
}


